@@ -449,7 +449,7 @@ locate_pythons_for_key(HKEY root, REGSAM flags)
 }
 
 static void
-locate_store_pythons()
+locate_store_pythons(void)
 {
 #if defined(_M_X64)
     /* 64bit process, so look in native registry */
@@ -466,7 +466,7 @@ locate_store_pythons()
 }
 
 static void
-locate_venv_python()
+locate_venv_python(void)
 {
     static wchar_t venv_python[MAX_PATH];
     INSTALLED_PYTHON * ip;
@@ -495,7 +495,7 @@ locate_venv_python()
 }
 
 static void
-locate_all_pythons()
+locate_all_pythons(void)
 {
     /* venv Python is highest priority */
     locate_venv_python();
@@ -694,7 +694,7 @@ static wchar_t wrapped_script_path[MAX_PATH];
  * valid wrapped script file.
  */
 static void
-locate_wrapped_script()
+locate_wrapped_script(void)
 {
     wchar_t * p;
     size_t plen;
@@ -770,8 +770,7 @@ run_child(wchar_t * cmdline)
     window, or fetching a message).  As this launcher doesn't do this
     directly, that cursor remains even after the child process does these
     things.  We avoid that by doing a simple post+get message.
-    See http://bugs.python.org/issue17290 and
-    https://bitbucket.org/vinay.sajip/pylauncher/issue/20/busy-cursor-for-a-long-time-when-running
+    See http://bugs.python.org/issue17290
     */
     MSG msg;
 
@@ -1035,7 +1034,7 @@ read_config_file(wchar_t * config_path)
     }
 }
 
-static void read_commands()
+static void read_commands(void)
 {
     if (launcher_ini_path[0])
         read_config_file(launcher_ini_path);
@@ -1267,7 +1266,11 @@ static PYC_MAGIC magic_values[] = {
     { 3390, 3399, L"3.7" },
     { 3400, 3419, L"3.8" },
     { 3420, 3429, L"3.9" },
-    { 3430, 3439, L"3.10" },
+    { 3430, 3449, L"3.10" },
+    /* Allow 50 magic numbers per version from here on */
+    { 3450, 3499, L"3.11" },
+    { 3500, 3549, L"3.12" },
+    { 3550, 3599, L"3.13" },
     { 0 }
 };
 
@@ -1682,7 +1685,7 @@ wcsdup_pad(const wchar_t *s, int padding, int *newlen)
 }
 
 static wchar_t *
-get_process_name()
+get_process_name(void)
 {
     DWORD bufferLen = MAX_PATH;
     DWORD len = bufferLen;
@@ -1923,27 +1926,35 @@ process(int argc, wchar_t ** argv)
         if (!cch) {
             error(0, L"Cannot determine memory for home path");
         }
-        cch += (DWORD)wcslen(PYTHON_EXECUTABLE) + 1 + 1; /* include sep and null */
+        cch += (DWORD)wcslen(PYTHON_EXECUTABLE) + 4; /* include sep, null and quotes */
         executable = (wchar_t *)malloc(cch * sizeof(wchar_t));
         if (executable == NULL) {
             error(RC_NO_MEMORY, L"A memory allocation failed");
         }
-        cch_actual = MultiByteToWideChar(CP_UTF8, 0, start, len, executable, cch);
+        /* start with a quote - we'll skip this ahead, but want it for the final string */
+        executable[0] = L'"';
+        cch_actual = MultiByteToWideChar(CP_UTF8, 0, start, len, &executable[1], cch - 1);
         if (!cch_actual) {
             error(RC_BAD_VENV_CFG, L"Cannot decode home path in '%ls'",
                   venv_cfg_path);
         }
+        cch_actual += 1; /* account for the first quote */
+        executable[cch_actual] = L'\0';
         if (executable[cch_actual - 1] != L'\\') {
             executable[cch_actual++] = L'\\';
             executable[cch_actual] = L'\0';
         }
-        if (wcscat_s(executable, cch, PYTHON_EXECUTABLE)) {
+        if (wcscat_s(&executable[1], cch - 1, PYTHON_EXECUTABLE)) {
             error(RC_BAD_VENV_CFG, L"Cannot create executable path from '%ls'",
                   venv_cfg_path);
         }
-        if (GetFileAttributesW(executable) == INVALID_FILE_ATTRIBUTES) {
+        /* there's no trailing quote, so we only have to skip one character for the test */
+        if (GetFileAttributesW(&executable[1]) == INVALID_FILE_ATTRIBUTES) {
             error(RC_NO_PYTHON, L"No Python at '%ls'", executable);
         }
+        /* now append the final quote */
+        wcscat_s(executable, cch, L"\"");
+        /* smuggle our original path through */
         if (!SetEnvironmentVariableW(L"__PYVENV_LAUNCHER__", argv0)) {
             error(0, L"Failed to set launcher environment");
         }
