@@ -3,7 +3,7 @@ import os.path
 import re
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import IO, Any, Dict, List, Optional, Set, Text, Tuple
+from typing import IO, Any
 
 from pegen import grammar
 from pegen.grammar import (
@@ -86,13 +86,13 @@ BASE_NODETYPES = {
 @dataclass
 class FunctionCall:
     function: str
-    arguments: List[Any] = field(default_factory=list)
-    assigned_variable: Optional[str] = None
-    assigned_variable_type: Optional[str] = None
-    return_type: Optional[str] = None
-    nodetype: Optional[NodeTypes] = None
+    arguments: list[Any] = field(default_factory=list)
+    assigned_variable: str | None = None
+    assigned_variable_type: str | None = None
+    return_type: str | None = None
+    nodetype: NodeTypes | None = None
     force_true: bool = False
-    comment: Optional[str] = None
+    comment: str | None = None
 
     def __str__(self) -> str:
         parts = []
@@ -124,14 +124,14 @@ class CCallMakerVisitor(GrammarVisitor):
     def __init__(
         self,
         parser_generator: ParserGenerator,
-        exact_tokens: Dict[str, int],
-        non_exact_tokens: Set[str],
+        exact_tokens: dict[str, int],
+        non_exact_tokens: set[str],
     ):
         self.gen = parser_generator
         self.exact_tokens = exact_tokens
         self.non_exact_tokens = non_exact_tokens
-        self.cache: Dict[Any, FunctionCall] = {}
-        self.cleanup_statements: List[str] = []
+        self.cache: dict[Any, FunctionCall] = {}
+        self.cleanup_statements: list[str] = []
 
     def keyword_helper(self, keyword: str) -> FunctionCall:
         return FunctionCall(
@@ -167,7 +167,7 @@ class CCallMakerVisitor(GrammarVisitor):
                 )
             return FunctionCall(
                 assigned_variable=f"{name.lower()}_var",
-                function=f"_PyPegen_expect_token",
+                function="_PyPegen_expect_token",
                 arguments=["p", name],
                 nodetype=NodeTypes.GENERIC_TOKEN,
                 return_type="Token *",
@@ -199,7 +199,7 @@ class CCallMakerVisitor(GrammarVisitor):
             type = self.exact_tokens[val]
             return FunctionCall(
                 assigned_variable="_literal",
-                function=f"_PyPegen_expect_token",
+                function="_PyPegen_expect_token",
                 arguments=["p", type],
                 nodetype=NodeTypes.GENERIC_TOKEN,
                 return_type="Token *",
@@ -286,7 +286,7 @@ class CCallMakerVisitor(GrammarVisitor):
             type = self.exact_tokens[val]
             return FunctionCall(
                 assigned_variable="_literal",
-                function=f"_PyPegen_expect_forced_token",
+                function="_PyPegen_expect_forced_token",
                 arguments=["p", type, f'"{val}"'],
                 nodetype=NodeTypes.GENERIC_TOKEN,
                 return_type="Token *",
@@ -298,7 +298,7 @@ class CCallMakerVisitor(GrammarVisitor):
             call.comment = None
             return FunctionCall(
                 assigned_variable="_literal",
-                function=f"_PyPegen_expect_forced_result",
+                function="_PyPegen_expect_forced_result",
                 arguments=["p", str(call), f'"{node.node.rhs!s}"'],
                 return_type="void *",
                 comment=f"forced_token=({node.node.rhs!s})",
@@ -374,10 +374,10 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
     def __init__(
         self,
         grammar: grammar.Grammar,
-        tokens: Dict[int, str],
-        exact_tokens: Dict[str, int],
-        non_exact_tokens: Set[str],
-        file: Optional[IO[Text]],
+        tokens: dict[int, str],
+        exact_tokens: dict[str, int],
+        non_exact_tokens: set[str],
+        file: IO[str] | None,
         debug: bool = False,
         skip_actions: bool = False,
     ):
@@ -388,7 +388,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
         self._varname_counter = 0
         self.debug = debug
         self.skip_actions = skip_actions
-        self.cleanup_statements: List[str] = []
+        self.cleanup_statements: list[str] = []
 
     def add_level(self) -> None:
         self.print("if (p->level++ == MAXSTACK) {")
@@ -424,12 +424,12 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
         self.print(f"if ({error_var}) {{")
         with self.indent():
             self.print(f"goto {goto_target};")
-        self.print(f"}}")
+        self.print("}")
 
     def out_of_memory_return(
         self,
         expr: str,
-        cleanup_code: Optional[str] = None,
+        cleanup_code: str | None = None,
     ) -> None:
         self.print(f"if ({expr}) {{")
         with self.indent():
@@ -438,14 +438,14 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
             self.print("p->error_indicator = 1;")
             self.print("PyErr_NoMemory();")
             self.add_return("NULL")
-        self.print(f"}}")
+        self.print("}")
 
     def out_of_memory_goto(self, expr: str, goto_target: str) -> None:
         self.print(f"if ({expr}) {{")
         with self.indent():
             self.print("PyErr_NoMemory();")
             self.print(f"goto {goto_target};")
-        self.print(f"}}")
+        self.print("}")
 
     def generate(self, filename: str) -> None:
         self.collect_rules()
@@ -488,8 +488,8 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
         if trailer:
             self.print(trailer.rstrip("\n") % dict(mode=mode, modulename=modulename))
 
-    def _group_keywords_by_length(self) -> Dict[int, List[Tuple[str, int]]]:
-        groups: Dict[int, List[Tuple[str, int]]] = {}
+    def _group_keywords_by_length(self) -> dict[int, list[tuple[str, int]]]:
+        groups: dict[int, list[tuple[str, int]]] = {}
         for keyword_str, keyword_type in self.keywords.items():
             length = len(keyword_str)
             if length in groups:
@@ -581,10 +581,10 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 self.print("if (_raw == NULL || p->mark <= _resmark)")
                 with self.indent():
                     self.print("break;")
-                self.print(f"_resmark = p->mark;")
+                self.print("_resmark = p->mark;")
                 self.print("_res = _raw;")
             self.print("}")
-            self.print(f"p->mark = _resmark;")
+            self.print("p->mark = _resmark;")
             self.add_return("_res")
         self.print("}")
         self.print(f"static {result_type}")
@@ -640,7 +640,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
             if memoize:
                 self.print("int _start_mark = p->mark;")
             self.print("void **_children = PyMem_Malloc(sizeof(void *));")
-            self.out_of_memory_return(f"!_children")
+            self.out_of_memory_return("!_children")
             self.print("Py_ssize_t _children_capacity = 1;")
             self.print("Py_ssize_t _n = 0;")
             if any(alt.action and "EXTRA" in alt.action for alt in rhs.alts):
@@ -658,7 +658,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                     self.add_return("NULL")
                 self.print("}")
             self.print("asdl_seq *_seq = (asdl_seq*)_Py_asdl_generic_seq_new(_n, p->arena);")
-            self.out_of_memory_return(f"!_seq", cleanup_code="PyMem_Free(_children);")
+            self.out_of_memory_return("!_seq", cleanup_code="PyMem_Free(_children);")
             self.print("for (int i = 0; i < _n; i++) asdl_seq_SET_UNTYPED(_seq, i, _children[i]);")
             self.print("PyMem_Free(_children);")
             if memoize and node.name:
@@ -712,7 +712,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
         self.print(call)
 
     def visit_Rhs(
-        self, node: Rhs, is_loop: bool, is_gather: bool, rulename: Optional[str]
+        self, node: Rhs, is_loop: bool, is_gather: bool, rulename: str | None
     ) -> None:
         if is_loop:
             assert len(node.alts) == 1
@@ -731,7 +731,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 self.visit(item)
         self.print(")")
 
-    def emit_action(self, node: Alt, cleanup_code: Optional[str] = None) -> None:
+    def emit_action(self, node: Alt, cleanup_code: str | None = None) -> None:
         self.print(f"_res = {node.action};")
 
         self.print("if (_res == NULL && PyErr_Occurred()) {")
@@ -773,7 +773,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
     def emit_dummy_action(self) -> None:
         self.print("_res = _PyPegen_dummy_name(p);")
 
-    def handle_alt_normal(self, node: Alt, is_gather: bool, rulename: Optional[str]) -> None:
+    def handle_alt_normal(self, node: Alt, is_gather: bool, rulename: str | None) -> None:
         self.join_conditions(keyword="if", node=node)
         self.print("{")
         # We have parsed successfully all the conditions for the option.
@@ -793,10 +793,10 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 self.emit_default_action(is_gather, node)
 
             # As the current option has parsed correctly, do not continue with the rest.
-            self.print(f"goto done;")
+            self.print("goto done;")
         self.print("}")
 
-    def handle_alt_loop(self, node: Alt, is_gather: bool, rulename: Optional[str]) -> None:
+    def handle_alt_loop(self, node: Alt, is_gather: bool, rulename: str | None) -> None:
         # Condition of the main body of the alternative
         self.join_conditions(keyword="while", node=node)
         self.print("{")
@@ -820,7 +820,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 self.print(
                     "void **_new_children = PyMem_Realloc(_children, _children_capacity*sizeof(void *));"
                 )
-                self.out_of_memory_return(f"!_new_children", cleanup_code="PyMem_Free(_children);")
+                self.out_of_memory_return("!_new_children", cleanup_code="PyMem_Free(_children);")
                 self.print("_children = _new_children;")
             self.print("}")
             self.print("_children[_n++] = _res;")
@@ -828,7 +828,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
         self.print("}")
 
     def visit_Alt(
-        self, node: Alt, is_loop: bool, is_gather: bool, rulename: Optional[str]
+        self, node: Alt, is_loop: bool, is_gather: bool, rulename: str | None
     ) -> None:
         if len(node.items) == 1 and str(node.items[0]).startswith("invalid_"):
             self.print(f"if (p->call_invalid_rules) {{ // {node}")
@@ -872,7 +872,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 self.print("}")
         self.print("}")
 
-    def collect_vars(self, node: Alt) -> Dict[Optional[str], Optional[str]]:
+    def collect_vars(self, node: Alt) -> dict[str | None, str | None]:
         types = {}
         with self.local_variable_context():
             for item in node.items:
@@ -880,7 +880,7 @@ class CParserGenerator(ParserGenerator, GrammarVisitor):
                 types[name] = type
         return types
 
-    def add_var(self, node: NamedItem) -> Tuple[Optional[str], Optional[str]]:
+    def add_var(self, node: NamedItem) -> tuple[str | None, str | None]:
         call = self.callmakervisitor.generate_call(node.item)
         name = node.name if node.name else call.assigned_variable
         if name is not None:
